@import url('https://fonts.googleapis.com/css2?family=IBM+Plex+Sans:ital,wght@0,400;0,500;0,600;0,700;1,400;1,500;1,600;1,700&display=swap');
@import url('https://fonts.googleapis.com/css2?family=IBM+Plex+Mono:ital,wght@0,400;0,500;0,600;0,700;1,400;1,500;1,600;1,700&display=swap');
@media all {
    body, input, div.sphinxsidebar input {
        font-size: 1.05rem;
    }

    code, kbd, pre, textarea {
        font-size: 1rem;
    }

    p.caption, .caption-text {
        font-weight: bold;
    }

    span.pre {
        font-family: var(--font-stack--monospace);
    }

    code.literal {
        font-size: 1rem;
    }

    .toc-title {
        font-weight: bold;
    }

    .toc-tree code.literal {
        font-size: 0.9rem;
    }

    div.highlight {
        border: 1px solid;
    }

    .bolditalic {
        font-style: italic;
        font-weight: bold;
    }

    ul.simple {
        margin-bottom: 1rem;
        margin-top: 0;
    }

    footer, .page-info .context, .bottom-of-page .left-details, .bottom-of-page .right-details {
        font-size: unset;
    }

    /* Hide "On GitHub" section from versions menu */
    div.injected > div.rst-versions > div.rst-other-versions > dl:nth-child(4) {
        display: none;
    }
}